#include "global.h"
#include "global.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include <util/delay.h>
#include <stdio.h>
#include <inttypes.h>

#include "uart.h"
#include "mcp2515.h"

#include "defaults.h"
#include "global.h"
#include "obd.h"

// ----------------------------------------------------------------------------

#define	PRINT(string, ...) printf_P(PSTR(string), ##__VA_ARGS__)

static int putchar__(char c, FILE *stream) {
	uart_putc(c);
	return 0;
}

static FILE mystdout = FDEV_SETUP_STREAM(putchar__, 0, _FDEV_SETUP_WRITE);

// ----------------------------------------------------------------------------
void print_can_message(tCAN *message)
{
	uint8_t length = message->header.length;
    // Кадр удаленного запроса (remote frame) — служит для запроса на передачу кадра данных с тем же идентификатором;
	PRINT("0x%3X %d %d ", message->id, message->header.rtr, length);
//	if (!message->header.rtr) {
	for (uint8_t i = 0; i < length; i++) PRINT(" 0x%02X", message->data[i]);
    PRINT("\r\n");
//	}
}



int main(void)
{
	uart_init(UART_BAUD_SELECT(57600UL, F_CPU));
	sei();
   
	stdout = &mystdout; // Перенаправление вывода

	if (!mcp2515_init()) {
		PRINT("ERR: MCP2515_NO_RESPOND! BYE!\r\n");
		for (;;);
	}

	

	mcp2515_bit_modify(CANCTRL, (1<<REQOP2)|(1<<REQOP1)|(1<<REQOP0), 0);

    
    tCAN message;
    
	while (1) {

        
        for (uint8_t speed_kmH = 0; speed_kmH < 250; speed_kmH=speed_kmH+10)
            for (uint16_t engine_RPM = 0; engine_RPM < 8500; engine_RPM=engine_RPM+100)
        
        
        {
                //_delay_ms(300);
                uint8_t speed_code = speed_kmH / 2.5;
                PRINT("Speed=%d km/H Code=%d \r\n", speed_kmH, speed_code);
                message.id = 0x1F9;
                message.header.length = 8;
                message.header.rtr = 0;
                message.data[0] = 0x00;
                message.data[1] = 0x00;
                message.data[2] = speed_code;
                message.data[3] = 0x00;
                message.data[4] = 0x00;
                message.data[5] = 0x00;
                message.data[6] = 0x00;
                message.data[7] = 0x00;
                
                PRINT("T ");
				print_can_message(&message);
                
                mcp2515_send_message(&message);
                
                _delay_ms(50);
                
                
                
                uint8_t RPM_code = engine_RPM / 32;
                PRINT("RPM=%d  Code=%d \r\n", engine_RPM, RPM_code);
                message.id = 0x180;
                message.header.length = 8;
                message.header.rtr = 0;
                message.data[0] = RPM_code; // allow up to 255
                message.data[1] = 0x00;
                message.data[2] = 0x00;
                message.data[3] = 0x00;
                message.data[4] = 0x00;
                message.data[5] = 0x00;
                message.data[6] = 0x00;
                message.data[7] = 0x00;
                
                PRINT("T ");
				print_can_message(&message);
                
                mcp2515_send_message(&message);
                
                _delay_ms(50);
                
                
                uint8_t FAILS = 0x00; // Аварии. Нет аварий.
                uint8_t AKB = 0x00;   // Норма заряда АКБ. Нормы нет.

//                 FAILS |= 1<<0; // Зажечь индикатор Check Engine постоянно
//                 FAILS |= 1<<1; // Моргать индикатором Check Engine 1 Гц.
//                 FAILS |= 1<<2; // Зажечь индикатор - нет давления масла
//                 FAILS |= 1<<3; // Зажечь индикатор - перегрев!

                AKB |= 1<<0; // Тушим лампочку аварии заряда АКБ

                message.id = 0x551;
                message.header.length = 8;
                message.header.rtr = 0;

                message.data[0] = 0x00; // Никакой реакции. Младший байт температуры двигателя?
                message.data[1] = 0x00; // Никакой реакции, кроме 0xFF - начнет моргать лампочкой перегрева 1Гц. Старший байт температуры?
                message.data[2] = 0x00; // Ничего не делает, вольтаж?
                message.data[3] = AKB; // Норма заряда, может еще и вольтаж?
                message.data[4] = FAILS; //Аварии
                message.data[5] = 0x00; // Никакой реакции. COUNTER.
                message.data[6] = 0x00; // Никакой реакции.
                message.data[7] = 0x00; // Никакой реакции.

                PRINT("T ");
				print_can_message(&message);
                
                mcp2515_send_message(&message);
                
                _delay_ms(50);
                
            }

            
            
            

            
            
            
            
            
            
            
            
            
	} 

	return 0;
}
