// #include "ecu_config.h"
#include <avr/io.h>
#include <avr/cpufunc.h>
#include <util/delay.h>
// #include "spi_dev.h"
#include "mcp2515.h"
#include "obd.h"



 /* Details from http://en.wikipedia.org/wiki/OBD-II_PIDs */
#define MODE1               0x01        //Show current data
#define MODE2               0x02        //Show freeze frame data
#define MODE3               0x03        //Show stored Diagnostic Trouble Codes
#define MODE4               0x04        //Clear Diagnostic Trouble Codes and stored values

#define PID_SUPPORTED       0x00
#define MONITOR_STATUS      0x01
#define ENGINE_COOLANT_TEMP 0x05
#define ENGINE_RPM          0x0C
#define VEHICLE_SPEED       0x0D
#define MAF_SENSOR          0x10
#define THROTTLE            0x11
#define O2_VOLTAGE          0x14

#define MODE1_RESPONSE      0x41
#define MODE3_RESPONSE      0x43
#define MODE4_RESPONSE      0x44
#define PID_REQUEST         0x7DF
#define PID_REPLY           0x7E8

#define SET_OBD_CS(v) WRITEBIT(PORTB, PB3, v)     //!< Switches chip into diagnostic mode

//can_t msg; // Create a test messsage

// -----------------------------------------------------------------------------
/** Set filters and masks.
 *
 * The filters are divided in two groups:
 *
 * Group 0: Filter 0 and 1 with corresponding mask 0.
 * Group 1: Filter 2, 3, 4 and 5 with corresponding mask 1.
 *
 * If a group mask is set to 0, the group will receive all messages.
 *
 * If you want to receive ONLY 11 bit identifiers, set your filters
 * and masks as follows:
 */
// 	uint8_t can_filter[]  = { //PROGMEM
// 		// Group 0
// 		MCP2515_FILTER(0),				// Filter 0
// 		MCP2515_FILTER(0),				// Filter 1
//
// 		// Group 1
// 		MCP2515_FILTER(0),				// Filter 2
// 		MCP2515_FILTER(0),				// Filter 3
// 		MCP2515_FILTER(0),				// Filter 4
// 		MCP2515_FILTER(0),				// Filter 5
//
// 		MCP2515_FILTER(0),				// Mask 0 (for group 0)
// 		MCP2515_FILTER(0),				// Mask 1 (for group 1)
// 	};

/*
 * If you want to receive ONLY 29 bit identifiers, set your filters
 * and masks as follows:
 *
 * \code
 *	uint8_t can_filter[] PROGMEM = {
 *		// Group 0
 *		MCP2515_FILTER_EXTENDED(0),		// Filter 0
 *		MCP2515_FILTER_EXTENDED(0),		// Filter 1
 *
 *		// Group 1
 *		MCP2515_FILTER_EXTENDED(0),		// Filter 2
 *		MCP2515_FILTER_EXTENDED(0),		// Filter 3
 *		MCP2515_FILTER_EXTENDED(0),		// Filter 4
 *		MCP2515_FILTER_EXTENDED(0),		// Filter 5
 *
 *		MCP2515_FILTER_EXTENDED(0),		// Mask 0 (for group 0)
 *		MCP2515_FILTER_EXTENDED(0),		// Mask 1 (for group 1)
 *	};
 * \endcode
 *
 * If you want to receive both 11 and 29 bit identifiers, set your filters
 * and masks as follows:
 */
// const uint8_t can_filter[] PROGMEM =
// {
// 	// Group 0
// 	MCP2515_FILTER(0),				// Filter 0
// 	MCP2515_FILTER(0),				// Filter 1
//
// 	// Group 1
// 	MCP2515_FILTER_EXTENDED(0),		// Filter 2
// 	MCP2515_FILTER_EXTENDED(0),		// Filter 3
// 	MCP2515_FILTER_EXTENDED(0),		// Filter 4
// 	MCP2515_FILTER_EXTENDED(0),		// Filter 5
//
// 	MCP2515_FILTER(0),				// Mask 0 (for group 0)
// 	MCP2515_FILTER_EXTENDED(0),		// Mask 1 (for group 1)
// };
// You can receive 11 bit identifiers with either group 0 or 1.




// void process_obd_interface(struct ecudata_t* d)
// {
//     if (mcp2515_check_message())
//     {
//         if (mcp2515_get_message(&msg))
//         {
//             // If we received a message resend it with a different id
//             //msg.id += 0x40;
// 				// Send the new message
//                 can_request(&msg, d);
// 				mcp2515_send_message(&msg);
//         }
//         s_timer_set(send_obd_interval_counter, OBD_TIME_INTERVAL);
//     } else if (s_timer_is_action(send_obd_interval_counter)) //периодически проверяем таймер OBD
//     {   // Рассылка, если не было адресного запроса
//         msg.id = 0x7DF;
//         can_request(&msg, d);
//         s_timer_set(send_obd_interval_counter, OBD_TIME_INTERVAL);
//     }
// }

// Message received!
// [00:42:30:262] id: 0x7df
// [00:42:30:262] laenge: 8
// [00:42:30:262] rtr: 0
// [00:42:30:262] daten: 0x02 0x01 0x00 0x00 0x00 0x00 0x00 0x00

void can_request( tCAN *msg, struct ecudata_t* d) // tCAN?
{
    if ( msg->id == 0x7DF ) //PID_REQUEST - запрос от сканера 0x7DF..0x7E7, а ответы от 7E8h до 7EFh т.е. 8 ECUs )
    {
        // MODE1 (0x01) Show current data
        // Считывание текущих параметров ЭСУД
        if ( msg->data[1] == MODE1 )
        {
            switch ( msg->data[2] ) /* Details from http://en.wikipedia.org/wiki/OBD-II_PIDs */
            {
                case 0x00: //PID_SUPPORTED:
                    msg->data[0] = 0x06;    // 6 bytes - длина ответа
                    msg->data[2] = 0x00;    //PID_SUPPORTED;
                    msg->data[3] = 0xE8;
                    msg->data[4] = 0x19;
                    msg->data[5] = 0x30;
                    msg->data[6] = 0x12;
                    msg->data[5] = 0x00;
                    break;

                case 0x01: // MONITOR_STATUS:
                    msg->data[0] = 0x05;  // Длина ответа
                    msg->data[2] = 0x01;  // MONITOR_STATUS;
                    //if(ecu.dtc == true) can_msg[3] = 0x82;
                    //    else
                    msg->data[3] = 0x00;
                    msg->data[4] = 0x07;
                    msg->data[5] = 0xFF;
                    break;

                case ENGINE_RPM:  //0x0C            //   ((A*256)+B)/4    [RPM]
                    msg->data[0] = 0x04;   //Длина ответа
                    msg->data[2] = ENGINE_RPM; //0x0C
                    msg->data[3] = (d->sens.inst_frq & 0xff00) >> 8;
                    msg->data[4] = d->sens.inst_frq & 0x00ff;
                    break;

                case ENGINE_COOLANT_TEMP: //0x05    //     A-40              [degree C]
                    msg->data[0] = 0x03;   //Длина ответа
                    msg->data[2] = ENGINE_COOLANT_TEMP; //0x05
                    msg->data[3] = d->sens.temperat;
                    break;

                case VEHICLE_SPEED:  //0x0D       // A                  [km]
                    msg->data[0] = 0x03;  //Длина ответа
                    msg->data[2] = VEHICLE_SPEED;
                    msg->data[3] = d->sens.speed;
                    break;

                case MAF_SENSOR:  //0x10             // ((256*A)+B) / 100  [g/s]
                    msg->data[0] = 0x04;  //Длина ответа
                    msg->data[2] = MAF_SENSOR;
                    msg->data[3] = (d->sens.map & 0xff00) >> 8; // TODO Not actual
                    msg->data[4] =  d->sens.map & 0x00ff;
                    break;

                case O2_VOLTAGE:            // A * 0.005   (B-128) * 100/128 (if B==0xFF, sensor is not used in trim calc)
                    msg->data[0] = 0x04;
                    msg->data[2] = O2_VOLTAGE;
                    msg->data[3] = 0;//d->corr.lambda & 0x00ff;
                    msg->data[4] = 0; //(d->corr.lambda & 0xff00) >> 8;
                    break;;

                case THROTTLE:            //
                    msg->data[0] = 0x03;
                    msg->data[2] = THROTTLE;
                    msg->data[3] = d->sens.tps;
                    break;
              }//switch

        } else
        // MODE3 (0x03) Show stored Diagnostic Trouble Codes — Считывание хранящихся кодов неисправностей
        if ( msg->data[1] == MODE3 ) // Request trouble codes //Show stored Diagnostic
        {
//          if(ce_state.ecuerrors == 0x00) // Типа нет ошибок
//          {
//              msg->data[0] = 0x02;
//              msg->data[2] = 0x00;
//          } else {
            msg->data[0] = 0x06;    // Типа ошибки
            msg->data[2] = 0x02;
            msg->data[3] = 0x01;
            msg->data[4] = 0x00;
            msg->data[5] = 0x02;
            msg->data[6] = 0x00;
        } else

        //#define MODE4               0x04        //Clear Diagnostic Trouble Codes and stored values
        //0x04. Clear Diagnostic Trouble Codes and stored values — Стирание кодов неисправностей, фотографий текущих параметров, результатов тестов датчиков кислорода, результатов тестовых мониторов.
        if ( msg->data[1] == MODE4 ) // Clear trouble codes, clear Check engine light
        {
            // TODO Сбросить ошибки
            //ecu.dtc = false;
            msg->data[0] = 0x00; //Длина ответа
        } // else шо шо??

        msg->id = 0x7E8; // Reply ECU 1
        msg->data[1] += 0x40; //for RESPONSE mode + 40

   } // else неизвестный запрос?

}


/*



#define OBD_TEST_REQUEST 0x7DF

#define OBD_TEST_RESPONSE 0x7E8

#define OBD_CURRENT_DATA 1
#define OBD_STORED_DIAGNOSTIC_TROUBLE_CODES 3
#define OBD_PENDING_DIAGNOSTIC_TROUBLE_CODES 7

#define PID_SUPPORTED_PIDS_REQUEST 0
#define PID_ENGINE_LOAD 0x04
#define PID_COOLANT_TEMP 0x05
#define PID_FUEL_PRESSURE 0x0A
#define PID_INTAKE_MAP 0x0B
#define PID_RPM 0x0C
#define PID_SPEED 0x0D
#define PID_TIMING_ADVANCE 0x0E
#define PID_INTAKE_TEMP 0x0F
#define PID_THROTTLE 0x11





#define OBD_TEST_REQUEST 0x7DF

#define OBD_TEST_RESPONSE 0x7E8

#define OBD_CURRENT_DATA 1
#define OBD_STORED_DIAGNOSTIC_TROUBLE_CODES 3
#define OBD_PENDING_DIAGNOSTIC_TROUBLE_CODES 7

#define PID_SUPPORTED_PIDS_REQUEST 0
#define PID_ENGINE_LOAD 0x04
#define PID_COOLANT_TEMP 0x05
#define PID_FUEL_PRESSURE 0x0A
#define PID_INTAKE_MAP 0x0B
#define PID_RPM 0x0C
#define PID_SPEED 0x0D
#define PID_TIMING_ADVANCE 0x0E
#define PID_INTAKE_TEMP 0x0F
#define PID_THROTTLE 0x11



extern CANTxFrame txmsg;




void setTxBit(int offset, int index) {

	txmsg.data8[offset] = txmsg.data8[offset] | (1 << index);

}



static void handleGetDataRequest(CANRxFrame *rx) {
	if (rx->data8[2] == PID_SUPPORTED_PIDS_REQUEST) //0
	{
		//scheduleMsg(&logger, "Got lookup request");

		commonTxInit(OBD_TEST_RESPONSE); //0x7E8 7E8 через 7EF
		txmsg.data8[0] = 6; // 6 bytes
		txmsg.data8[1] = 0x41; // mode 1
 MODE1_RESPONSE
		txmsg.data8[2] = PID_SUPPORTED_PIDS_REQUEST; // 0


// 		PID = 0x01 Монитор состояния с момента очистки DTC. т.е. Лампа индикации неисправности (MIL),
// 		статус и количество кодов неисправности.) Побитовая кодировка.
// 		Возвращает 4 байта данных

		txmsg.data8[3] |= 1 << 7;

// 		PID = 0x02 Обнаруженные диагностические коды ошибок
// 		Возвращает 2 байта данных
		txmsg.data8[3] |= 1 << 6;

// 		PID = 0x03 Состояние топливной системы. Побитовая кодировка.
// 		Возвращает 2 байта данных
		txmsg.data8[3] |= 1 << 5;

// 		PID = 0x04 Расчетное значение нагрузки на двигатель. A*100/255
// 		Возвращает 1 байт данных
		txmsg.data8[3] |= 1 << 4;

// 		PID = 0x05 Температура охлаждающей жидкости °C. A-40
// 		Возвращает 1 байт данных
		txmsg.data8[3] |= 1 << 3;

// 		PID = 0x06 Степень обеднения впрыска, или того Short, Bank 1 (A-128) * 100/128
// 		Возвращает 1 байт данных
		txmsg.data8[3] |= 1 << 2;

// 		PID = 0x07 Степень обеднения впрыска, или того Long, Bank 1 (A-128) * 100/128
// 		Возвращает 1 байт данных
		txmsg.data8[3] |= 1 << 1;

// 		PID = 0x08 Степень обеднения впрыска, или того Short, Bank 2 (A-128) * 100/128
// 		Возвращает 1 байт данных
		txmsg.data8[3] |= 1 << 0;


		///////////////////////////////////////////////////////////////////


// 		PID = 0x09 Степень обеднения впрыска, или того Long, Bank 2 (A-128) * 100/128
// 		Возвращает 1 байт данных
		txmsg.data8[4] |= 1 << 7;

// 		PID = 0x0A Давление топлива (кПа) A*3
// 		Возвращает 1 байт данных
		txmsg.data8[4] |= 1 << 6;

// 		PID = 0x0B Давление во впускном коллекторе (абсолютное) (кПа) A
// 		Возвращает 1 байт данных
		txmsg.data8[4] |= 1 << 5;

// 		PID = 0x0C Обороты двигателя (об/мин) ((A*256)+B)/4
// 		Возвращает 2 байт данных
		txmsg.data8[4] |= 1 << 4;

// 		PID = 0x0D Скорость автомобиля (км/ч) A
// 		Возвращает 1 байт данных
		txmsg.data8[4] |= 1 << 3;

// 		PID = 0x0E Угол опережения зажигания ° A/2 — 64
// 		Возвращает 1 байт данных
		txmsg.data8[4] |= 1 << 2;

// 		PID = 0x0F Температура всасываемого воздуха A-40
// 		Возвращает 1 байт данных
		txmsg.data8[4] |= 1 << 1;

// 		PID = 0x10 Массовый расход воздуха (грамм/сек) ((A*256)+B) / 100
// 		Возвращает 2 байта данных
		txmsg.data8[4] |= 1 << 0;

		//////////////////////////////////////////////


// 		PID = 0x11 Положение дроссельной заслонки % A*100/255
// 		Возвращает 1 байт данных
		txmsg.data8[5] |= 1 << 7;

// 		PID = 0x12 Commanded secondary air status. ? (2 банка по 4 датчика). Побитовая кодировка.
// 		Возвращает 1 байт данных
		txmsg.data8[5] |= 1 << 6;

// 		PID = 0x13 Наличие датчиков кислорода. (2 банка по 4 датчика). Побитовая кодировка.
// 		[A0..A3] == Bank 1, Sensors 1-4. [A4..A7] == Bank 2,Sensors 1-4.
// 		Возвращает 1 байт данных
		txmsg.data8[5] |= 1 << 5;


// 		PID = 0x14 Датчика кислорода из банка 1 номер 1. Старший байт - напряжение (A/200).
// 		Младший байт - степень коррекции (100/128)*B-100. Если FF - не прогрелся, не учитывается.
// 		Возвращает 2 байта данных
		txmsg.data8[5] |= 1 << 4;

// 		PID = 0x15 Датчика кислорода из банка 1 номер 2. Старший байт - напряжение (A/200).
// 		Младший байт - степень коррекции (100/128)*B-100. Если FF - не прогрелся, не учитывается.
// 		Возвращает 2 байта данных
		txmsg.data8[5] |= 1 << 3;

// 		PID = 0x16 Датчика кислорода из банка 1 номер 3. Старший байт - напряжение (A/200).
// 		Младший байт - степень коррекции (100/128)*B-100. Если FF - не прогрелся, не учитывается.
// 		Возвращает 2 байта данных
		txmsg.data8[5] |= 1 << 2;

// 		PID = 0x17 Датчика кислорода из банка 1 номер 4. Старший байт - напряжение (A/200).
// 		Младший байт - степень коррекции (100/128)*B-100. Если FF - не прогрелся, не учитывается.
// 		Возвращает 2 байта данных
		txmsg.data8[5] |= 1 << 1;

// 		PID = 0x18 Датчика кислорода из банка 2 номер 1. Старший байт - напряжение (A/200).
// 		Младший байт - степень коррекции (100/128)*B-100. Если FF - не прогрелся, не учитывается.
// 		Возвращает 2 байта данных
		txmsg.data8[5] |= 1 << 1;


// 		PID = 0x19 Датчика кислорода из банка 2 номер 2. Старший байт - напряжение (A/200).
// 		Младший байт - степень коррекции (100/128)*B-100. Если FF - не прогрелся, не учитывается.
// 		Возвращает 2 байта данных
		txmsg.data8[6] |= 1 << 7;

// 		PID = 0x1A Датчика кислорода из банка 2 номер 3. Старший байт - напряжение (A/200).
// 		Младший байт - степень коррекции (100/128)*B-100. Если FF - не прогрелся, не учитывается.
// 		Возвращает 2 байта данных
		txmsg.data8[6] |= 1 << 6;

// 		PID = 0x1B Датчика кислорода из банка 2 номер 4. Старший байт - напряжение (A/200).
// 		Младший байт - степень коррекции (100/128)*B-100. Если FF - не прогрелся, не учитывается.
// 		Возвращает 2 байта данных
		txmsg.data8[6] |= 1 << 5;


// 		PID = 0x1C Какой стандарт OBD поддерживает этот блок?
// 		Возвращает 1 байт данных
		txmsg.data8[6] |= 1 << 4;

// 		PID = 0x1D (Аналог PID = 0x13) Наличие датчиков кислорода. (2 банка по 4 датчика). Побитовая кодировка.
// 		[A0..A7] == [B1S1, B1S2, B2S1, B2S2, B3S1, B3S2, B4S1, B4S2]
// 		Возвращает 1 байт данных
		txmsg.data8[6] |= 1 << 3;

// 		PID = 0x1E Состояние внешних входов. Побитовая кодировка.
// 		Возвращает 1 байт данных
		txmsg.data8[6] |= 1 << 2;

// 		PID = 0x1F Время, прошедшее с момента недавнего запуска двигателя (A*256)+B
// 		Возвращает 2 байта данных
		txmsg.data8[6] |= 1 << 1;

// 		PID = 0x20 Список поддерживаемых PID’ов (21-40)
// 		Возвращает 4 байта данных
		txmsg.data8[6] |= 1 << 0;

        //      Другой пример уведомления
        //		setTxBit(3, 8 - PID_ENGINE_LOAD); //0x04
        //		setTxBit(3, 8 - PID_COOLANT_TEMP); //0x05
        //		setTxBit(4, 16 - PID_RPM); //0x0C
        //		setTxBit(4, 16 - PID_TIMING_ADVANCE); //0x0E
        //		setTxBit(4, 16 - PID_INTAKE_TEMP); //0x0F
        //		setTxBit(5, 24 - PID_THROTTLE); //PID_THROTTLE 0x11

		sendMessage();
	} else if (rx->data8[2] == PID_COOLANT_TEMP) // 0x05
	{
		scheduleMsg(&logger, "Got CLT request");

		int clt = 67; // todo: use real value

		commonTxInit(OBD_TEST_RESPONSE); //0x7E8 //7E0h до 7E7h
		txmsg.data8[0] = 3; // 3 bytes
		txmsg.data8[1] = 0x41; // mode 1
		txmsg.data8[2] = PID_COOLANT_TEMP; //0x05
		txmsg.data8[3] = clt + 40;
		sendMessage();
	} else if (rx->data8[2] == PID_RPM) // 0x0C
	{
		scheduleMsg(&logger, "Got RPM request");
		unsigned int rpm = 2000*4; //	rotation/min.	((A*256)+B)/4 todo: use real value
		commonTxInit(OBD_TEST_RESPONSE); //0x7E8
		txmsg.data8[0] = 4; // 4 bytes
		txmsg.data8[1] = 0x41; // mode 1
		txmsg.data8[2] = PID_RPM; //0x0C
		txmsg.data8[3] = ((rpm>>8) & 0xFF);
		txmsg.data8[4] = ((rpm) & 0xFF);
		sendMessage();

	} else if (rx->data8[2] == PID_TIMING_ADVANCE) {
		scheduleMsg(&logger, "Got timing request");
	} else if (rx->data8[2] == PID_SPEED) {
		scheduleMsg(&logger, "Got speed request");
		commonTxInit(OBD_TEST_RESPONSE); //0x7E8
		txmsg.data8[0] = 3; // 3 bytes
		txmsg.data8[1] = 0x41; // mode 1
		txmsg.data8[2] = PID_INTAKE_MAP; // 0x0B
		txmsg.data8[3] = 123; // todo: real speed
		sendMessage();
	} else if (rx->data8[2] == PID_FUEL_PRESSURE) //0x0A
	{
		scheduleMsg(&logger, "Got fuel pressure request");
	} else if (rx->data8[2] == PID_INTAKE_MAP) //0x0B
	{
		scheduleMsg(&logger, "Got MAP request");
		commonTxInit(OBD_TEST_RESPONSE); //0x7E8
		txmsg.data8[0] = 3; // 3 bytes
		txmsg.data8[1] = 0x41; // mode 1
		txmsg.data8[2] = PID_INTAKE_MAP; //0x0B
		txmsg.data8[3] = 46; // todo: real MAP
		sendMessage();
	} else {
		scheduleMsg(&logger, "Got unhandled request");
	}

}

void obdOnCanPacketRx(CANRxFrame *rx) {
	if (rx->SID != 0x7DF ) // Широковещательный запрос 0x7DF OBD_TEST_REQUEST
	{
		return;
	}
	if (rx->data8[0] == 2 && rx->data8[1] == OBD_CURRENT_DATA) { //1
		handleGetDataRequest(rx);
	} else
        if (rx->data8[0] == 1 && rx->data8[1] == OBD_STORED_DIAGNOSTIC_TROUBLE_CODES) {
		scheduleMsg(&logger, "Got stored DTC request");
	} else {
		scheduleMsg(&logger, "Got unhandled OBD message");
	}
}



bool OBD2processRequest(uint8_t mode, uint8_t pid, char *inData, char *outData)
{
	bool ret = false;
	switch (mode) {
		case 1: //show current data
			ret = processShowData(pid, inData, outData);
			outData[1] = mode + 0x40;
			outData[2] = pid;
			break;
		case 2: //show freeze frame data - not sure we'll be supporting this
			break;
		case 3: //show stored diagnostic codes - we can probably map our faults to some existing DTC codes or roll our own
			break;
		case 4: //clear diagnostic trouble codes - If we get this frame we just clear all codes no questions asked.
			break;
		case 6: //test results over CANBus (this replaces mode 5 from non-canbus) - I know nothing of this
			break;
		case 7: //show pending diag codes (current or last driving cycle) - Might just overlap with mode 3
			break;
		case 8: //control operation of on-board systems - this sounds really proprietary and dangerous. Maybe ignore this?
			break;
		case 9: //request vehicle info - We can identify ourselves here but little else
			break;
		case 0x20: //custom PID codes we made up for GEVCU
			break;
	}
	return ret;
}

//Process SAE standard PID requests. Function returns whether it handled the request or not.
bool OBD2processShowData(uint8_t pid, char *inData, char *outData) {
	int temp;


	switch (pid) {
	case 0: //pids 1-0x20 that we support - bitfield
		//returns 4 bytes so immediately indicate that.
		outData[0] = 4;
		outData[3] = 0b11011000; //pids 1 - 8 - starting with pid 1 in the MSB and going from there
		outData[4] = 0b00010000; //pids 9 - 0x10
		outData[5] = 0b10000000; //pids 0x11 - 0x18
		outData[6] = 0b00010011; //pids 0x19 - 0x20
		return true;
		break;
	case 1: //Returns 32 bits but we really can only support the first byte which has bit 7 = Malfunction? Bits 0-6 = # of DTCs
		outData[0] = 4;
		outData[3] = 0; //TODO: We aren't properly keeping track of faults yet but when we do fix this.
		outData[4] = 0; //these next three are really related to ICE diagnostics
		outData[5] = 0; //so ignore them.
		outData[6] = 0;
		return true;
		break;
	case 2: //Freeze DTC
		return false; //don't support freeze framing yet. Might be useful in the future.
		break;
	case 4: //Calculated engine load (A * 100 / 255) - Percentage
		temp = (255 * motorController->getTorqueActual()) / motorController->getTorqueAvailable();
		outData[0] = 1;
		outData[3] = (uint8_t)(temp & 0xFF);
		return true;
		break;
	case 5: //Engine Coolant Temp (A - 40) = Degrees Centigrade
		//our code stores temperatures as a signed integer for tenths of a degree so translate
		temp =  motorController->getTemperatureSystem() / 10;
		if (temp < -40) temp = -40;
		if (temp > 215) temp = 215;
		temp += 40;
		outData[0] = 1; //returning only one byte
		outData[3] = (uint8_t)(temp);
		return true;
		break;
	case 0xC: //Engine RPM (A * 256 + B) / 4
		temp = motorController->getSpeedActual() * 4; //we store in RPM while the PID code wants quarter rpms
		outData[0] = 2;
		outData[3] = (uint8_t)(temp / 256);
		outData[4] = (uint8_t)(temp);
		return true;
		break;
	case 0x11: //Throttle position (A * 100 / 255) - Percentage
		temp = motorController->getThrottle() / 10; //getThrottle returns in 10ths of a percent
		if (temp < 0) temp = 0; //negative throttle can't be shown for OBDII
		temp = (255 * temp) / 100;
		outData[0] = 1;
		outData[3] = (uint8_t)(temp);
		return true;
		break;
	case 0x1C: //Standard supported (We return 1 = OBDII)
		outData[0] = 1;
		outData[3] = 1;
		return true;
		break;
	case 0x1F: //runtime since engine start (A*256 + B)
		outData[0] = 2;
		outData[3] = 0; //TODO: Get the actual runtime.
		outData[4] = 0;
		return true;
		break;
	case 0x20: //pids supported (next 32 pids - formatted just like PID 0)
		outData[0] = 4;
		outData[3] = 0b10000000; //pids 0x21 - 0x28 - starting with pid 0x21 in the MSB and going from there
		outData[4] = 0b00000010; //pids 0x29 - 0x30
		outData[5] = 0b00000000; //pids 0x31 - 0x38
		outData[6] = 0b00000001; //pids 0x39 - 0x40
		return true;
		break;
	case 0x21: //Distance traveled with fault light lit (A*256 + B) - In km
		outData[0] = 2;
		outData[3] = 0; //TODO: Can we get this information?
		outData[4] = 0;
		return true;
		break;
	case 0x2F: //Fuel level (A * 100 / 255) - Percentage
		outData[0] = 1;
		outData[3] = 0; //TODO: finish BMS interface and get this value
		return true;
		break;
	case 0x40: //PIDs supported, next 32
		outData[0] = 4;
		outData[3] = 0b00000000; //pids 0x41 - 0x48 - starting with pid 0x41 in the MSB and going from there
		outData[4] = 0b00000000; //pids 0x49 - 0x50
		outData[5] = 0b10000000; //pids 0x51 - 0x58
		outData[6] = 0b00000001; //pids 0x59 - 0x60
		return true;
		break;
	case 0x51: //What type of fuel do we use? (We use 8 = electric, presumably.)
		outData[0] = 1;
		outData[3] = 8;
		return true;
		break;
	case 0x60: //PIDs supported, next 32
		outData[0] = 4;
		outData[3] = 0b11100000; //pids 0x61 - 0x68 - starting with pid 0x61 in the MSB and going from there
		outData[4] = 0b00000000; //pids 0x69 - 0x70
		outData[5] = 0b00000000; //pids 0x71 - 0x78
		outData[6] = 0b00000000; //pids 0x79 - 0x80
		return true;
		break;
	case 0x61: //Driver requested torque (A-125) - Percentage
		temp = (100 * motorController->getTorqueRequested()) / motorController->getTorqueAvailable();
		temp += 125;
		outData[0] = 1;
		outData[3] = (uint8_t)temp;
		return true;
		break;
	case 0x62: //Actual Torque delivered (A-125) - Percentage
		temp = (100 * motorController->getTorqueActual()) / motorController->getTorqueAvailable();
		temp += 125;
		outData[0] = 1;
		outData[3] = (uint8_t)temp;
		return true;
		break;
	case 0x63: //Reference torque for engine - presumably max torque - A*256 + B - Nm
		temp = motorController->getTorqueAvailable();
		outData[0] = 2;
		outData[3] = (uint8_t)(temp / 256);
		outData[4] = (uint8_t)(temp & 0xFF);
		return true;
		break;
	}
	return false;
}*/

// ECU#1 0x7E8 (2024)
// Vehicle Identification Number XTA21907777777777
// Calibration Verification Number ???
//
//
//
// $03 Fuel System status
// $04 Calculated engine load
// $05 Engine Coolant Temperature
// $06 Short Term Fuel Trim - Bank 1
// $07 Long Term Fuel Trim - Bank 1
// $0C Engine RPM
// $0D Vehicle Speed Sensor
// $0E Ignition Timing Advance for #1 Cylinder
// $0F Intake Air Temperature
// $10 Air Flow Rate from Mass Flow Sensor
// $11 Absolute Throttle Position
// $13 Location of Oxygen Sensors
// $14 Bank 1 - Sensor 1 (O2S)
// $15 Bank 1 - Sensor 2 (O2S)
// $1C OBD requirements to which vehicle is certified
// $21 Distance Traveled while MIL is Activated
// $F0 Fuel consumption
// $F4 Battery voltage
//
//
// ECU#2 0x7EC (2028)
// $F4 Battery voltage
