#ifndef __obd_h_included__
#define __obd_h_included__

#define OBD_TIME_INTERVAL    20

typedef struct sensors_t
{
 uint16_t map;                           //!< Intake Manifold Pressure (давление во впускном коллекторе (усредненное))
 uint16_t voltage;                       //!< Board voltage (напряжение бортовой сети (усредненное))
#ifdef SEND_INST_VAL
 uint16_t inst_voltage;                  //!< Instant valtage - not averaged
 uint16_t inst_map;                      //!< Intake Manifold Pressure - not averaged
 uint16_t inst_add_i1;                   //!< ADD_I1 input voltage - not averaged
#endif
 int16_t  temperat;                      //!< Coolant temperature (температура охлаждающей жидкости (усредненная))
 uint16_t frequen;                       //!< Averaged RPM (частота вращения коленвала (усредненная))
 uint16_t inst_frq;                      //!< Instant RPM - not averaged  (мгновенная частота вращения)
 uint8_t  carb;                          //!< State of carburetor's limit switch (состояние концевика карбюратора)
 uint8_t  gas;                           //!< State of gas valve (состояние газового клапана)
 uint16_t knock_k;                       //!< Knock signal level (уровень сигнала детонации)
 uint8_t  tps;                           //!< Throttle position sensor (0...100%, x2)
 uint16_t add_i1;                        //!< ADD_I1 input voltage
 uint16_t add_i2;                        //!< ADD_I2 input voltage
//#ifdef SPEED_SENSOR
 uint16_t speed;                         //!< Vehicle speed expressed by period between speed sensor pulses (1 tick = 4us)
 uint32_t distance;                      //!< Distance expressed by number of speed sensor pulses since last ignition turn on
//#endif
#ifdef AIRTEMP_SENS
 int16_t air_temp;                       //!< Intake air temperature
#endif
#ifdef PA4_INP_IGNTIM
 uint16_t pa4;                           //!< PA4 input voltage
#endif

#if defined(FUEL_INJECT) || defined(GD_CONTROL)
 int16_t tpsdot;                         //!< Speed of TPS movement (d%/dt = %/s), positive when acceleration, negative when deceleration
#endif

 //сырые значения датчиков (дискреты АЦП с компенсированными погрешностями)
 int16_t  map_raw;                       //!< raw ADC value from MAP sensor
 int16_t  voltage_raw;                   //!< raw ADC value from voltage
 int16_t  temperat_raw;                  //!< raw ADC value from coolant temperature sensor
 int16_t  tps_raw;                       //!< raw ADC value from TPS sensor
 int16_t  add_i1_raw;                    //!< raw ADC value from ADD_I1 input
 int16_t  add_i2_raw;                    //!< raw ADC value from ADD_I2 input
}sensors_t;


/**Describes system's data (main ECU data structure)
 * описывает данные системы, обеспечивает единый интерфейс данных
 */
typedef struct ecudata_t
{
//  struct params_t  param;                 //!< --parameters (параметры)
  struct sensors_t sens;                  //!< --sensors (сенсоры)
//  struct correct_t corr;                  //!< --calculated corrections and lookup tables' values

 uint8_t  ie_valve;                      //!< State of Idle cut off valve (состояние клапана ЭПХХ)
 uint8_t  fe_valve;                      //!< State of Power valve (состояние клапана ЭМР)
#if defined(FUEL_INJECT) || defined(GD_CONTROL)
 uint8_t  fc_revlim;                     //!< Flag indicates fuel cut from rev. limitter
#endif
 uint8_t  cool_fan;                      //!< State of the cooling fan (состояние электровентилятора)
 uint8_t  st_block;                      //!< State of the starter blocking output (состояние выхода блокировки стартера)
 uint8_t  ce_state;                      //!< State of CE lamp (состояние лампы "CE")
 uint8_t  airflow;                       //!< Air flow (расход воздуха)
 uint8_t  choke_pos;                     //!< Choke position in % * 2
 uint8_t  gasdose_pos;       /*GD*/      //!< Gas dosator position in % * 2

#ifdef REALTIME_TABLES
 f_data_t tables_ram;                    //!< set of tables in RAM
 mm_func8_ptr_t mm_ptr8;                 //!< callback, returns 8 bit result, unsigned
 mm_func16_ptr_t mm_ptr16;               //!< callback, return 16 bit result, unsigned
#endif
//  f_data_t _PGM *fn_dat;                  //!< Pointer to the set of tables (указатель на набор характеристик)

 uint16_t op_comp_code;                  //!< Contains code of operation for packet being sent - OP_COMP_NC (содержит код который посылается через UART (пакет OP_COMP_NC))
 uint16_t op_actn_code;                  //!< Contains code of operation for packet being received - OP_COMP_NC (содержит код который принимается через UART (пакет OP_COMP_NC))
 uint16_t ecuerrors_for_transfer;        //!< Buffering of error codes being sent via UART in real time (буферизирует коды ошибок передаваемые через UART в реальном времени)
 uint16_t ecuerrors_saved_transfer;      //!< Buffering of error codes for read/write from/to EEPROM which is being sent/received (буферизирует коды ошибок для чтения/записи в EEPROM, передаваемые/принимаемые через UART)
 uint8_t  use_knock_channel_prev;        //!< Previous state of knock channel's usage flag (предыдущее состояние признака использования канала детонации)

 //TODO: To reduce memory usage it is possible to use crc or some simple hash algorithms to control state of memory(changes). So this variable becomes unneeded.
 uint8_t* eeprom_parameters_cache;       //!< Pointer to an array of EEPROM parameters cache (reduces redundant write operations)

 uint8_t engine_mode;                    //!< Current engine mode(start, idle, work) (текущий режим двигателя (пуск, ХХ, нагрузка))

#ifdef DIAGNOSTICS
 diagnost_inp_t diag_inp;                //!< diagnostic mode: values of inputs
 uint16_t       diag_out;                //!< diagnostic mode: values of outputs
#endif

 uint8_t choke_testing;                  //!< Used to indcate that choke testing is on/off (so it is applicable only if SM_CONTROL compilation option is used)
 int8_t choke_manpos_d;                  //!< Muanual position setting delta value used for choke control
 uint8_t choke_rpm_reg;                  //!< Used to indicate that at the moment system regulates RPM by means of choke position

 uint8_t gasdose_testing;    /*GD*/      //!< Used to indcate that gas dosator testing is on/off (so it is applicable only if GD_CONTROL compilation option is used)
 int8_t gasdose_manpos_d;    /*GD*/      //!< Muanual position setting delta value used for gasdose control

 uint8_t bt_name[9];                     //!< received name for Bluetooth (8 chars max), zero element is size
 uint8_t bt_pass[7];                     //!< received password for Bluetooth (6 chars max), zero element is size
 uint8_t sys_locked;                     //!< used by immobilizer to indicate that system is locked

#ifdef FUEL_INJECT
 uint16_t inj_pw;                        //!< current value of injector pulse width
#endif

#if defined(FUEL_INJECT) || defined(GD_CONTROL)
 uint8_t acceleration;                   //!< acceleration/decelaration flag
#endif

}ecudata_t;


// void process_obd_interface(struct ecudata_t* d);

typedef struct
{
	#if SUPPORT_EXTENDED_CANID
		uint32_t id;				//!< ID der Nachricht (11 oder 29 Bit)
		struct {
			int rtr : 1;			//!< Remote-Transmit-Request-Frame?
			int extended : 1;		//!< extended ID?
		} flags;
	#else
		uint16_t id;				//!< ID der Nachricht (11 Bit)
		struct {
			int rtr : 1;			//!< Remote-Transmit-Request-Frame?
		} flags;
	#endif

	uint8_t length;				//!< Anzahl der Datenbytes
	uint8_t data[8];			//!< Die Daten der CAN Nachricht

// 	#if SUPPORT_TIMESTAMPS
// 		uint16_t timestamp;
// 	#endif
} can_t;

void can_request( tCAN *msg, struct ecudata_t* d);

// // typedef uint32_t canmbx_t;
//
// /**
//  * @brief   CAN transmission frame.
//  * @note    Accessing the frame data as word16 or word32 is not portable because
//  *          machine data endianness, it can be still useful for a quick filling.
//  */
// typedef struct {
//   /*lint -save -e46 [6.1] Standard types are fine too.*/
//   uint8_t                   DLC:4;          /**< @brief Data length.        */
//   uint8_t                   RTR:1;          /**< @brief Frame type.         */
//   uint8_t                   IDE:1;          /**< @brief Identifier type.    */
//   union {
//     uint32_t                SID:11;         /**< @brief Standard identifier.*/
//     uint32_t                EID:29;         /**< @brief Extended identifier.*/
//     uint32_t                _align1;
//   };
//   /*lint -restore*/
//   union {
//     uint8_t                 data8[8];       /**< @brief Frame data.         */
//     uint16_t                data16[4];      /**< @brief Frame data.         */
//     uint32_t                data32[2];      /**< @brief Frame data.         */
//   };
// } CANTxFrame;
//
// /**
//  * @brief   CAN received frame.
//  * @note    Accessing the frame data as word16 or word32 is not portable because
//  *          machine data endianness, it can be still useful for a quick filling.
//  */
// typedef struct {
//   /*lint -save -e46 [6.1] Standard types are fine too.*/
//   uint8_t                   FMI;            /**< @brief Filter id.          */
//   uint16_t                  TIME;           /**< @brief Time stamp.         */
//   uint8_t                   DLC:4;          /**< @brief Data length.        */
//   uint8_t                   RTR:1;          /**< @brief Frame type.         */
//   uint8_t                   IDE:1;          /**< @brief Identifier type.    */
//   union {
//     uint32_t                SID:11;         /**< @brief Standard identifier.*/
//     uint32_t                EID:29;         /**< @brief Extended identifier.*/
//     uint32_t                _align1;
//   };
//   /*lint -restore*/
//   union {
//     uint8_t                 data8[8];       /**< @brief Frame data.         */
//     uint16_t                data16[4];      /**< @brief Frame data.         */
//     uint32_t                data32[2];      /**< @brief Frame data.         */
//   };
// } CANRxFrame;
//
//







#endif //__obd_h_included__
